"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.bindMiddleware = bindMiddleware;
exports.serializeInput = serializeInput;
exports.deserializeOutput = deserializeOutput;
// Helper to compose middleware functions
const composeMiddleware = (fns) => {
    return (input) => __awaiter(void 0, void 0, void 0, function* () {
        let result = input;
        for (const fn of fns) {
            result = yield fn(result);
        }
        return result;
    });
};
// Get all deserialize functions in the correct order
const getDeserializeChain = (middleware) => {
    return middleware.map((m) => m.input.deserialize);
};
// Get all serialize functions in the correct order
const getSerializeChain = (middleware) => {
    return middleware.map((m) => m.input.serialize);
};
/**
 * Binds middleware to a workflow's tasks
 * @param wf - The workflow declaration to bind middleware to
 * @param client - The HatchetClient instance
 * @returns The workflow with middleware bound to its tasks
 */
function bindMiddleware(wf, client) {
    return __awaiter(this, void 0, void 0, function* () {
        const tasks = wf.definition._tasks.map((task) => {
            if (!task.middleware) {
                // eslint-disable-next-line no-param-reassign
                task.middleware = [
                    {
                        input: {
                            deserialize: (input) => input,
                            serialize: (input) => input,
                        },
                        output: {
                            deserialize: (input) => input,
                            serialize: (input) => input,
                        },
                    },
                ];
            }
            const originalFn = task.fn;
            // Build input middleware chain
            let inputDeserializeChain = [];
            let inputSerializeChain = [];
            if (task.middleware) {
                inputDeserializeChain = getDeserializeChain(task.middleware);
                inputSerializeChain = getSerializeChain(task.middleware);
            }
            // Build output middleware chain
            let outputDeserializeChain = [];
            let outputSerializeChain = [];
            if (task.middleware) {
                outputDeserializeChain = getDeserializeChain(task.middleware);
                outputSerializeChain = getSerializeChain(task.middleware);
            }
            // Add client middleware if present
            if (client.middleware) {
                inputDeserializeChain = [...getDeserializeChain(client.middleware), ...inputDeserializeChain];
                inputSerializeChain = [...inputSerializeChain, ...getSerializeChain(client.middleware)];
                outputDeserializeChain = [
                    ...getDeserializeChain(client.middleware),
                    ...outputDeserializeChain,
                ];
                outputSerializeChain = [...outputSerializeChain, ...getSerializeChain(client.middleware)];
            }
            // Compose the middleware with the original function
            // eslint-disable-next-line no-param-reassign
            task.fn = (input, ctx) => __awaiter(this, void 0, void 0, function* () {
                console.log('fn input', input);
                // Apply input deserialize chain
                const deserializedInput = yield composeMiddleware(inputDeserializeChain)(input);
                // Run the original function
                const result = yield originalFn(deserializedInput, ctx);
                // Apply output serialize chain
                return composeMiddleware(outputSerializeChain)(result);
            });
            return task;
        });
        // eslint-disable-next-line no-param-reassign
        wf.definition._tasks = tasks;
        return wf;
    });
}
function serializeInput(input, middleware) {
    return __awaiter(this, void 0, void 0, function* () {
        let serialized = input;
        if (middleware) {
            for (const m of middleware) {
                serialized = yield m.input.serialize(serialized);
            }
        }
        return serialized;
    });
}
function deserializeOutput(output, middleware) {
    return __awaiter(this, void 0, void 0, function* () {
        let deserialized = output;
        if (middleware) {
            for (const m of middleware) {
                deserialized = yield m.output.deserialize(deserialized);
            }
        }
        return deserialized;
    });
}
